# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-27 14:49
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0009_auto_20160623_1521'),
        ('budget', '0011_alter_print_run_date'),
    ]

    operations = [
    ]
