# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-03-28 18:47
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='package',
            name='headline_voting_open',
        ),
        migrations.AddField(
            model_name='package',
            name='headline_status',
            field=models.CharField(choices=[(b'drafting', b'Drafting'), (b'voting', b'Voting'), (b'finalized', b'Finalized')], default=b'drafting', max_length=9),
        ),
        migrations.AlterField(
            model_name='headlinevote',
            name='headline',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='headline_vote', to='budget.Headline'),
        ),
    ]
