# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-27 18:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0015_auto_20160627_1308'),
    ]

    operations = [
        migrations.RenameField(
            model_name='package',
            old_name='url',
            new_name='published_url',
        ),
        migrations.AlterField(
            model_name='headline',
            name='package',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='headline_candidates', to='budget.Package'),
        ),
    ]
