# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-18 16:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0025_auto_20170817_1931'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='slug',
            field=models.CharField(db_index=True, editable=False, max_length=100),
        ),
    ]
