<div class="center-content">
    <div class="edit-bar">
        <div id="presence-indicator">
            <div class="presence-notice-holder">
                <div class="presence-heading">
                    <div class="fa fa-exclamation-triangle neon"></div>
                    <h2>Caution</h2>
                </div>
                <div class="other-active-users"></div>
            </div>
        </div>
        <div class="button-holder">
            <div class="delete-trigger material-button flat-button"><span>Delete</span></div>
            <div class="primary-action save-trigger material-button flat-button"><span>Save</span></div>
            <div class="save-and-continue-editing-trigger material-button flat-button show-for-medium"><span>Save &amp; continue editing</span></div>
            <div class="clearer"></div>
        </div>
    </div>
    <div class="single-page">
        <div class="package-header">
            <div class="color-dot"></div>
            <h1>Add content</h1>
        </div>
        <form id="package-form">
            <div class="error-message"></div>

            <div class="row">
                <div class="medium-6 columns form-group">
                    <input id="hub" name="hub" class="to-selectize" type="text" placeholder="Choose an option" autocomplete="off" autocorrect="off" data-form="package" required="" isRequired="true" value="" />
                    <label for="hub" class="control-label">Hub<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-3 columns form-group">
                    <input id="type" name="type" class="to-selectize" type="text" placeholder="Choose an option" autocomplete="off" autocorrect="off" data-form="primary" value="" required="" isRequired="true" />
                    <label for="type" class="control-label">Type<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-3 columns form-group variable-attribute-group">
                    <div class="length-group variable-attribute">
                        <input id="length" name="length" data-form="primary" class="form-select-word-count" type="number" autocomplete="off" autocorrect="off" value="" required="" />
                        <label for="length" class="control-label">Word count</label>
                        <i class="bar"></i>
                    </div>

                    {% if visualsRequestURL %}
                    <div class="request-link-group variable-attribute">
                        <div class="request-button-holder">
                            <div class="request-link material-button flat-button"><a href="{{ visualsRequestURL }}" target="_blank"><span>Request visuals</span></a></div>
                        </div>
                    </div>
                    {% endif %}
                </div>
            </div>

            <div class="row">
                <div class="medium-3 columns form-group">
                    <input id="pub_date_resolution" name="pub_date_resolution" class="to-selectize" type="text" placeholder="Choose an option" autocomplete="off" autocorrect="off" data-form="package" required="" isRequired="true" />
                    <label for="pub_date_resolution" class="control-label">Time format<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-5 columns form-group pub-date-group">
                    <input id="pub_date" name="pub_date" data-form="package" type="text" autocomplete="off" autocorrect="off" required="" isRequired="true" value="" />
                    <label for="pub_date" class="control-label">Published date<span class="required-marker">*</span></label>
                    <span class="datepicker-trigger rich-widget-trigger" tabindex="0"><i class="material-icons md-24">today</i></span>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-4 columns form-group pub-time-group">
                    <input id="pub_time" name="pub_time" data-form="package" type="text" autocomplete="off" autocorrect="off" value="" />
                    <label for="pub_time" class="control-label">Published time&thinsp;*</label>
                    <span class="timepicker-trigger rich-widget-trigger" tabindex="0"><i class="material-icons md-24">access_time</i></span>
                    <i class="bar"></i>
                </div>
            </div>

            <div class="row">
                <div class="medium-12 columns form-group">
                    <div class="slug-group-holder">
                        <div class="hub-slug-value">hub.</div>
                        <div class="keyword-group">
                            <div class="keyword-value">keyword</div>
                            <input id="slug_key" name="slug_key" data-form="primary" type="text" placeholder="keyword" autocomplete="off" value="" required="" isRequired="true" maxlength="20" />
                        </div>
                        <div class="formatted-date-value">.date</div>
                        <div class="clearer"></div>
                    </div>
                    <label for="slug_key" class="control-label">Slug<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-6 columns"></div>
            </div>

            <div class="row">
                <!--  -->
                <div class="medium-12 columns form-group">
                    <div id="budget_line_holder" class="expanding-holder">
                        <textarea id="budget_line" name="budget_line" data-form="primary" rows="1" required="" isRequired="true" ></textarea>
                        <pre class="budget-spacer"><span></span><br></pre>
                    </div>
                    <label for="budget_line" class="control-label">Budget line<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
            </div>

            <div class="row">
                <div class="medium-6 columns form-group set-authors">
                    <input id="authors" name="authors" class="to-selectize staff-select reporter" data-form="primary" type="text" isRequired="true" value="" />
                    <label for="authors" class="control-label">Author(s)<span class="required-marker">*</span></label>
                    <i class="bar"></i>
                    <div class="form-help"></div>
                </div>
                <div class="medium-6 columns form-group set-editors">
                    <input id="editors" name="editors" class="to-selectize staff-select editor" data-form="primary" type="text" value="" />
                    <label for="editors" class="control-label">Editor(s)</label>
                    <i class="bar"></i>
                </div>
            </div>

            {% if showHeadlines %}
            <div class="row collapsible-row-header" data-expand-target="headlines">
                <h4>Headlines</h4>
            </div>
            {% endif %}

            <div class="row can-collapse" data-expand-receiver="headlines">
                <div id="headline-fields" class="collapsable-inner">
                    <div class="extra-spacing"></div>

                    <div class="hl-variable-group" data-mode="other">
                        <div class="medium-6 columns form-group">
                            <input id="headline1" name="headline1" data-form="package" type="text" value="" required="" />
                            <label for="headline1" class="control-label">Headline #1</label>
                            <i class="bar"></i>
                        </div>

                        <div class="medium-6 columns form-group">
                            <input id="headline2" name="headline2" data-form="package" type="text" value="" required="">
                            <label for="headline2" class="control-label">Headline #2</label>
                            <i class="bar"></i>
                        </div>

                        <div class="medium-6 columns form-group">
                            <input id="headline3" name="headline3" data-form="package" type="text" value="" required="" />
                            <label for="headline3" class="control-label">Headline #3</label>
                            <i class="bar"></i>
                        </div>

                        <div class="medium-6 columns form-group">
                            <input id="headline4" name="headline4" data-form="package" type="text" value="" required="" />
                            <label for="headline4" class="control-label">Headline #4</label>
                            <i class="bar"></i>
                        </div>

                        <div class="clearer"></div>
                    </div>

                    <div class="hl-variable-group medium-12 columns form-radio" data-mode="voting">
                        <div class="radio">
                            <label>
                                <input id="headlineRadio1" name="headlineChoices" data-form="package" type="radio" value="" /><i class="helper"></i><span class="radio-label"> <span class="vote-count">(<span class="vote-total">0</span><span class="percent-sign">%</span>)</span></span>
                            </label>
                        </div>

                        <div class="radio">
                            <label>
                                <input id="headlineRadio2" name="headlineChoices" data-form="package" type="radio" value="" /><i class="helper"></i><span class="radio-label"> <span class="vote-count">(<span class="vote-total">0</span><span class="percent-sign">%</span>)</span>
                            </label>
                        </div>

                        <div class="radio">
                            <label>
                                <input id="headlineRadio3" name="headlineChoices" data-form="package" type="radio" value="" /><i class="helper"></i><span class="radio-label"> <span class="vote-count">(<span class="vote-total">0</span><span class="percent-sign">%</span>)</span>
                            </label>
                        </div>

                        <div class="radio">
                            <label>
                                <input id="headlineRadio4" name="headlineChoices" data-form="package" type="radio" value="" /><i class="helper"></i><span class="radio-label"> <span class="vote-count">(<span class="vote-total">0</span><span class="percent-sign">%</span>)</span>
                            </label>
                        </div>

                        <div class="clearer"></div>
                    </div>

                    <div id="vote-submission-toggle" class="medium-12 columns checkbox">
                        <label>
                            <input id="headlinesReady" name="headlinesReady" data-form="package" type="checkbox" value="ready" /><i class="helper"></i>Submit headlines to a vote
                            <!-- <input type="checkbox" checked="" /><i class="helper"></i>Submit headlines to a vote -->
                        </label>
                    </div>

                    <div class="clearer"></div>
                </div>
            </div>

            <div class="clearer"></div>

            <div class="row collapsible-row-header" data-expand-target="production-notes">
                <h4>Production notes</h4>
            </div>

            <div class="row can-collapse" data-expand-receiver="production-notes">
                <div class="collapsable-inner">
                    <div class="medium-12 columns notes-reveal form-group">
                        <div id="notes-quill" class="quill-holder">
                        </div>
                    </div>
                    <div class="clearer"></div>
                </div>
            </div>

            <div class="clearer"></div>

            <div class="row collapsible-row-header" data-expand-target="publishing-details">
                <h4>Publishing details</h4>
            </div>

            <div class="row can-collapse" data-expand-receiver="publishing-details">
                <div id="publishing-fields" class="collapsable-inner">
                    <div class="extra-spacing"></div>

                    <div class="medium-12 columns form-group">
                        <input id="url" name="url" data-form="package" type="text" value="" />
                        <label for="url" class="control-label">URL</label>
                        <i class="bar"></i>
                    </div>

                    <div class="clearer"></div>
                </div>
            </div>

            <div class="clearer"></div>
        </form>
    </div>

    <div class="single-page">
        <div id="additional-forms">
            <div class="section-header">
                <h4 class="final-header">Additional content <span class="add-additional-content-trigger material-button"><i class="fa fa-plus"></i><span>&thinsp;Add<span class="show-for-medium">&nbsp;content</span></span></span></h4>
            </div>
        </div>

        <div id="additional-content-children" class="additional-content">
        </div>

        <div class="bottom-button-holder">
            <div class="add-additional-content-trigger material-button"><i class="fa fa-plus"></i><span>&thinsp;Add content</span></div>
        </div>
    </div>

    <div id="content-placements-table" class="single-page table-card">
        <div class="list-header">
            <h4>Print placements <span class="hint">(Click to edit)</span></h4>
            <div class="create-placement material-button flat-button"><span>Create new</span></div>
        </div>
        <div id="content-placements-loading">
            <div class="spinner"></div>
            <div class="loading-text">Loading...</div>
        </div>
        <div class="table-holder">
            <table class="full-results">
                <thead>
                    <tr>
                        <th class="destination">Destination</th>
                        <th class="run-date">Run date(s)</th>
                        <th class="external-slug">Slug</th>
                        <th class="placement-types">Type(s)</th>
                        <th class="page-and-details">Page / Details</th>
                        <th class="delete-action-holder">Delete</th>
                    </tr>
                </thead>
                <tbody>
                </tbody>
            </table>

            <div class="table-pagination">
                <div class="current-range">
                    <span class="this-page-range"><span class="range-start">0</span>&thinsp;&mdash;&thinsp;<span class="range-end">0</span></span> of <span class="total-records">0</span></div>
                    <a class="btn disabled btn-default btn-primary-flat prev-trigger" href="#" type="link">
                        <i class="material-icons md-24">keyboard_arrow_left</i>
                    </a>
                    <a class="btn disabled btn-default btn-primary-flat next-trigger" href="#" type="link">
                        <i class="material-icons md-24">keyboard_arrow_right</i>
                    </a>
            </div>
        </div>
    </div>
</div>
