# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-24 02:39
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0008_package_publish_date'),
    ]

    operations = [
        migrations.RenameField(
            model_name='package',
            old_name='print_run_date',
            new_name='print_run_date_old',
        ),
    ]
