# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-24 23:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0034_contentplacement_populate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='package',
            name='is_print_placement_finalized',
        ),
        migrations.RemoveField(
            model_name='package',
            name='print_placements',
        ),
        migrations.RemoveField(
            model_name='package',
            name='print_run_date',
        ),
        migrations.RemoveField(
            model_name='package',
            name='print_run_date_old',
        ),
        migrations.RemoveField(
            model_name='package',
            name='print_section',
        ),
        migrations.RemoveField(
            model_name='package',
            name='print_system_slug',
        ),
    ]
