# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-21 19:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('budget', '0030_auto_20170820_2132'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='length',
            field=models.PositiveIntegerField(blank=True, help_text='How long is this article?', null=True, verbose_name='Length (in words)'),
        ),
        migrations.AlterField(
            model_name='item',
            name='slug_key',
            field=models.CharField(blank=True, help_text="Will be used, with package info, to generate additional items' slugs.", max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='item',
            name='type',
            field=models.CharField(choices=[('text', 'Text'), ('photo', 'Photo'), ('video', 'Video'), ('audio', 'Audio'), ('graphic', 'Graphic'), ('interactive', 'Interactive'), ('aggregation', 'Aggregation'), ('other', 'Other')], default='text', max_length=25),
        ),
        migrations.AlterField(
            model_name='package',
            name='headline_status',
            field=models.CharField(choices=[('drafting', 'Drafting'), ('voting', 'Voting'), ('finalized', 'Finalized')], default='drafting', max_length=9),
        ),
        migrations.AlterField(
            model_name='package',
            name='pub_date_resolution',
            field=models.CharField(blank=True, choices=[('m', 'Month'), ('w', 'Week'), ('d', 'Day'), ('t', 'Time')], default='d', editable=False, max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='package',
            name='slug_key',
            field=models.CharField(help_text='Miniature package descriptor. Will be used to generate the slug.', max_length=25, null=True),
        ),
        migrations.AlterField(
            model_name='printpublication',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Should this publication be included in the current options users can choose from? If false, this publication will be archived, but it may be re-activated in the future.', verbose_name='Active?'),
        ),
        migrations.AlterField(
            model_name='printpublication',
            name='priority',
            field=models.PositiveSmallIntegerField(default=10, help_text='In the list of all active print publications, how high should this entry show up?'),
        ),
        migrations.AlterField(
            model_name='printsection',
            name='is_active',
            field=models.BooleanField(default=True, help_text='Should this publication be included in the current options users can choose from? If false, this publication will be archived, but it may be re-activated in the future.', verbose_name='Active?'),
        ),
        migrations.AlterField(
            model_name='printsection',
            name='priority',
            field=models.PositiveSmallIntegerField(default=10, help_text='In the list of all active print publications, how high should this entry show up?'),
        ),
    ]
