<tag class="white">{% for link in links %}<a href="{{ link.destination }}">{{ link.name }}</a>&nbsp;&thinsp;|&thinsp;&nbsp{% endfor %}{% if currentUser.email %}{{ currentUser.email }}{% else %}{{ currentUser.username }}{% endif %} <i class="fa fa-lock neon"></i></tag>
